


'''
	import legumes.recipe_with_goals.formulate as formulate_recipe_with_goals
	formulate_recipe_with_goals.beautifully (
		ingredients = [
			[ retrieve_supp ("coated tablets/multivitamin_276336.JSON"), 10 ]
		],
		goals = {}
	)
'''

'''
	fake data: {
		"info": {
			"includes": [],
			"names": [
				"protein"
			],
			"region": 1
		},
		"measures": {
			"mass + mass equivalents": {
				"per recipe": {
					"grams": {
						"fraction string": "65193545255861875341/11258999068426240"
					}
				},
				"portion of grove": {
					"fraction string": "1303870905117237506820000/3661863958435401439007099"
				}
			}
		},
		"goals": {
			"per day"
				"mass + mass equivalents": {
					"per recipe": {
							"grams": {
								"fraction string": "3/100000"
							}
					}
				}
			}
		}
	}
'''

'''
	{
		"labels": [ "Biotin" ],
		"goal": {
			"mass + mass equivalents": {
				"per day": {
					"grams": {
						"fraction string": "3/100000"
					}
				}
			}
		}
	}
'''

'''
	days = amount in recipe / amount of goal
'''

import ships.modules.exceptions.parse as parse_exception
import legumes.goals.human.FDA as human_FDA_goal
import apoplast.shows.ingredient_scan_recipe.formulate as formulate_recipe
import apoplast.measures.number.decimal.reduce as reduce_decimal
import apoplast.shows.ingredient_scan.DB.scan.seek as seek_nutrient

from .modules.find_goal import find_goal

import rich
from fractions import Fraction
import copy

def add_goals (
	essential_nutrients_grove,
	goals_ingredients,
	
	records = 0
):
	skipped_goal = []
	skipped_composition = []
	for ingredient in essential_nutrients_grove:
		assert ("info" in ingredient), ingredient
		assert ("names" in ingredient ["info"]), ingredient
	
		ingredient_names = ingredient ["info"] ["names"]
		ingredient ["goals"] = {}
		
		goal = None;
		try:
			goal = find_goal (
				ingredient_names = ingredient_names,
				goals_ingredients = goals_ingredients
			)
		except Exception as E:	
			if (records >= 1): print (parse_exception.now (E))
			skipped_goal.append (ingredient_names)
			continue;
		
		
		#rich.print_json (data = goal)
		
		#
		#	grams:
		#	
		try:
			if ("mass + mass equivalents" in ingredient ["measures"]):			
				grams_per_recipe = (
					ingredient ["measures"] ["mass + mass equivalents"] ["per recipe"] ["grams"] ["fraction string"]
				)
				grams_per_goal = (
					goal ["goal"] ["mass + mass equivalents"] ["per Earth day"] ["grams"] ["fraction string"]
				)
				goal_per_day = (
					Fraction (grams_per_recipe) / 
					Fraction (grams_per_goal)
				)
				
				decimal_string = "?"
				try:
					decimal_string = str (reduce_decimal.start (goal_per_day, partial_size = 3));
				except Exception:
					pass;
				
				ingredient ["goals"] = {
					"days of ingredient": {
						"mass + mass equivalents": {
							"per recipe": {
								"fraction string": str (goal_per_day),
								"decimal string": decimal_string
							}
						}
					}
				}
			
	
			
		except Exception as E:
			if (records >= 1): print (parse_exception.now (E))
			skipped_composition.append (ingredient_names)
			pass;
			
		
		if ("unites" in ingredient):
			#raise Exception ("unites")
		
			add_goals (
				ingredient ["unites"],
				goals_ingredients
			)

	return [
		skipped_goal,
		skipped_composition
	]

def beautifully (
	recipe = None,
	ingredients = [],
	goals = {},
	
	records = 0
):
	if (type (recipe) != dict):
		recipe = formulate_recipe.adroitly (
			ingredients
		)
		
	essential_nutrients = recipe ["essential nutrients"]
	essential_nutrients_grove = essential_nutrients ["grove"]
	cautionary_ingredients = recipe ["cautionary ingredients"]
	
	if (
		type (goals) != dict or 
		(type (goals) == dict and "ingredients" not in goals)
	):
		return {
			"recipe": recipe,
			"skipped_composition": [],
			"skipped_goal": [],
			"note": "goal ingredients not found"
		};
		
	
	recipe ["goals"] = goals	
	goals_ingredients = goals ["ingredients"];
	
	[ skipped_goal, skipped_composition ] = add_goals (
		essential_nutrients_grove,
		goals_ingredients
	)
	

	return {
		"recipe": recipe,
		"skipped_composition": skipped_composition,
		"skipped_goal": skipped_goal
	};
	
	




'''
	import legumes.goals.human.FDA as human_FDA_goal
	goal = human_FDA_goal.retrieve ()
'''

'''
	https://ods.od.nih.gov/HealthInformation/nutrientrecommendations.aspx
'''

'''
	https://www.fda.gov/food/nutrition-facts-label/daily-value-nutrition-and-supplement-facts-labels
'''

#
#	 of 4 or more years
#

'''
{
	"labels": [ "Added sugars" ],
	"goal": {
		"mass + mass equivalents": {
			"per day": {
				"grams": {
					"fraction string": "50/1"
				}
			}
		}
	}
},
'''

'''
	equivalents:
		{
			"labels": [ "Folate", "Folic Acid" ],
			"goal": [ "400", "mcg", "DFE" ],
		}
		
		{
			"labels": [ "Niacin" ],
			"goal": [ "16", "mg", "NE" ],
		}
		
		{
			"labels": [ "Vitamin A" ],
			"goal": [ "900", "mcg", "RAE" ],
		}
		
		{
			"labels": [ "Vitamin E" ],
			"goal": [ "15", "mg", "alpha-tocopherol" ],
		}
'''

def retrieve ():
	return {
		"label": "FDA goals for average adult humans",
		"cautions": [
			"These guidelines have not been checked by any high status nutritionists.",
			"Please consult with your actual physician or nutritionist also."
		],
		"ingredients": [
			
			{
				"labels": [ "Biotin" ],
				"goal": [ "30", "mcg" ],
			},
			{
				"labels": [ "Calcium" ],
				"goal": [ "1300", "mg" ],
			},
			{
				"labels": [ "calories" ],
				"goal": [ "2000", "kcal" ],
			},
			{
				"labels": [ "Choline" ],
				"goal": [ "550", "mg" ],
			},
			{
				"labels": [ "Cholesterol" ],
				"goal": [ "300", "mg" ],
			},
			{
				"labels": [ "Chromium" ],
				"goal": [ "35", "mcg" ],
			},
			{
				"labels": [ "Copper" ],
				"goal": [ "0.9", "mg" ],
			},
			{
				"labels": [ "Dietary Fiber" ],
				"goal": [ "28", "g" ],
			},
			{
				"labels": [ "Fat" ],
				"goal": [ "78", "g" ],
			},
			{
				"labels": [ "Folate", "Folic Acid" ],
				"goal": [ "400", "mcg", "DFE" ],
			},
			{
				"labels": [ "Iodine" ],
				"goal": [ "150", "mcg" ],
			},
			{
				"labels": [ "Iron" ],
				"goal": [ "18", "mg" ],
			},
			{
				"labels": [ "Magnesium" ],
				"goal": [ "420", "mg" ],
			},
			{
				"labels": [ "Manganese" ],
				"goal": [ "2.3", "mg" ],
			},
			{
				"labels": [ "Molybdenum" ],
				"goal": [ "45", "mcg" ],
			},
			{
				"labels": [ "Niacin" ],
				"goal": [ "16", "mg", "NE" ],
			},
			{
				"labels": [ "Pantothenic Acid" ],
				"goal": [ "5", "mg" ],
			},
			{
				"labels": [ "Phosphorus" ],
				"goal": [ "1250", "mg" ],
			},
			{
				"labels": [ "Potassium" ],
				"goal": [ "4700", "mg" ],
			},
			{
				"labels": [ "Protein" ],
				"goal": [ "50", "g" ],
			},
			{
				"labels": [ "Riboflavin" ],
				"goal": [ "1.3", "mg" ],
			},
			{
				"labels": [ "Saturated Fat" ],
				"goal": [ "20", "g" ],
			},
			{
				"labels": [ "Selenium" ],
				"goal": [ "55", "mcg" ],
			},
			{
				"labels": [ "Sodium" ],
				"goal": [ "2300", "mg" ],
			},
			{
				"labels": [ "Thiamin" ],
				"goal": [ "1.2", "mg" ],
			},
			{
				"labels": [ "Total carbohydrate" ],
				"goal": [ "275", "g" ],
			},
			{
				"labels": [ "Vitamin A" ],
				"goal": [ "900", "mcg", "RAE" ],
			},
			{
				"labels": [ "Vitamin B6" ],
				"goal": [ "1.7", "mg" ],
			},
			{
				"labels": [ "Vitamin B12" ],
				"goal": [ "2.4", "mcg" ],
			},
			{
				"labels": [ "Vitamin C" ],
				"goal": [ "90", "mcg" ],
			},
			{
				"labels": [ "Vitamin D" ],
				"goal": [ "20", "mcg" ],
			},
			{
				"labels": [ "Vitamin E" ],
				"goal": [ "15", "mg", "alpha-tocopherol" ],
			},
			{
				"labels": [ "Vitamin K" ],
				"goal": [ "120", "mcg" ],
			},
			{
				"labels": [ "Zinc" ],
				"goal": [ "11", "mg" ],
			},
		],
		"limiters": [
			{
				"label": "species",
				"includes": [
					"human"
				]               
			},
			{
				"kind": "slider--integer",
				"label": "age",
				"includes": [
					[
						"4",
						"eternity"
					]
				],
			},
			{
				"label": "exclusions",
				"includes": [
					"pregnant",
					"breast feeding"
				],
			}
		],
		"sources": [
			"https://www.fda.gov/food/new-nutrition-facts-label/daily-value-new-nutrition-and-supplement-facts-labels",
			"https://www.fda.gov/food/nutrition-facts-label/calories-nutrition-facts-label",
			"https://www.fda.gov/media/99069/download",
			"https://www.fda.gov/media/99059/download"
		]
	}












'''
	python3 status.proc.py recipe_with_goals/_status/0_1_basics/status_1.py
'''


import apoplast.clouds.food_USDA.deliveries.one.assertions.foundational as assertions_foundational
import apoplast.clouds.food_USDA.examples as USDA_examples	
import apoplast.clouds.food_USDA.nature as food_USDA_nature
import apoplast.insure.equality as equality

import apoplast.shows.ingredient_scan_recipe.formulate as formulate_recipe
import apoplast.shows.ingredient_scan.grove.seek_name_or_accepts as grove_seek_name_or_accepts

import apoplast.clouds.supp_NIH.nature as supp_NIH_nature
import apoplast.clouds.supp_NIH.examples as NIH_examples

import rich

from fractions import Fraction
from copy import deepcopy
import json

def find_grams (measures):
	return Fraction (
		measures ["mass + mass equivalents"] ["per recipe"] ["grams"] ["fraction string"]
	)

def add (path, data):
	import pathlib
	from os.path import dirname, join, normpath
	this_directory = pathlib.Path (__file__).parent.resolve ()
	example_path = normpath (join (this_directory, path))
	FP = open (example_path, "w")
	FP.write (data)
	FP.close ()
	
def retrieve_supp (supp_path):
	return supp_NIH_nature.create (
		NIH_examples.retrieve (supp_path) 
	)

def retrieve_food (food_path):
	return food_USDA_nature.create (
		USDA_examples.retrieve (food_path)
	)

def check_1 ():
	recipe = formulate_recipe.adroitly ([
		[ retrieve_supp ("coated tablets/multivitamin_276336.JSON"), 10 ]
	])
	
	add ("status_1.JSON", json.dumps (recipe, indent = 4))
	
	assert (len (recipe ["essential nutrients"] ["natures"]) == 1)

	rich.print_json (data = recipe)
	
checks = {
	"check 1": check_1
}








'''
	This has a simultaneous DB lookup issue perhaps...?
'''




'''
	python3 status.proc.py recipe_with_goals/_status/1_1_recipe/status_1.py
'''


#
#	foods
#
import apoplast.clouds.food_USDA.deliveries.one.assertions.foundational as assertions_foundational
import apoplast.clouds.food_USDA.examples as USDA_examples	
import apoplast.clouds.food_USDA.nature as food_USDA_nature

#
#	supps
#
import apoplast.clouds.supp_NIH.nature as supp_NIH_nature
import apoplast.clouds.supp_NIH.examples as NIH_examples

import apoplast.insure.equality as equality

import apoplast.shows.ingredient_scan_recipe.formulate as formulate_recipe
import apoplast.shows.ingredient_scan.grove.seek_name_or_accepts as grove_seek_name_or_accepts


import legumes.recipe_with_goals.formulate as formulate_recipe_with_goals
import legumes.goals.human.FDA as human_FDA_goal
	
import rich

from fractions import Fraction
from copy import deepcopy
import json

def find_grams (measures):
	return Fraction (
		measures ["mass + mass equivalents"] ["per recipe"] ["grams"] ["fraction string"]
	)
	
def add (path, data):
	import pathlib
	from os.path import dirname, join, normpath
	this_directory = pathlib.Path (__file__).parent.resolve ()
	example_path = normpath (join (this_directory, path))
	FP = open (example_path, "w")
	FP.write (data)
	FP.close ()
	
def retrieve_supp (supp_path):
	return supp_NIH_nature.create (
		NIH_examples.retrieve (supp_path) 
	)

def retrieve_food (food_path):
	return food_USDA_nature.create (
		USDA_examples.retrieve (food_path)
	)

def check_1 ():
	
		
	
		
	'''
	recipe = formulate_recipe.adroitly ([
		[ retrieve_supp ("coated tablets/multivitamin_276336.JSON"), 10 ]
	])
	'''
		
	recipe_with_goals = formulate_recipe_with_goals.beautifully (
		ingredients = [
			[ retrieve_supp ("coated tablets/multivitamin_276336.JSON"), 10 ]
		],
		goals = human_FDA_goal.retrieve ()
	)
	
	add (
		"status_1.JSON", 
		json.dumps (recipe_with_goals, indent = 4)
	)
	
	
	'''	
		per recipe:				3/100 * 90 * 10 = 27
		per day:				3/200
		
		days of ingredient: 	(27) / (3/200) = 1800
	'''
	
	
	
	vitamin_E = grove_seek_name_or_accepts.politely (
		grove = recipe_with_goals ["recipe"] ["essential nutrients"] ["grove"],
		name_or_accepts = "vitamin e"
	)
	rich.print_json (data = vitamin_E)
	assert (
		vitamin_E ["goals"] ["days of ingredient"] ["mass + mass equivalents"] ["per recipe"] ["fraction string"] == 
		"1800"
	), vitamin_E ["goals"]
	
	
	'''
		per recipe:				.050 * 90 * 10 = 45
			per form: .050
		
		per day:				2/125
		
		days of ingredient:		45 / (2/125) = 2812.5 = 5625/2
	'''
	vitamin_B3 = grove_seek_name_or_accepts.politely (
		grove = recipe_with_goals ["recipe"] ["essential nutrients"] ["grove"],
		name_or_accepts = "vitamin b3"
	)
	rich.print_json (data = vitamin_B3)
	assert (
		vitamin_B3 ["goals"] ["days of ingredient"] ["mass + mass equivalents"] ["per recipe"] ["fraction string"] == 
		"5625/2"
	), vitamin_B3 ["goals"] ["days of ingredient"] ["mass + mass equivalents"] ["per recipe"] ["fraction string"]
	
checks = {
	"check 1": check_1
}







'''
	find_goal (
		ingredient_names = [ 
			"protein" 
		],
		goals_ingredients = [{
			"labels": [
			  "Vitamin E"
			],
			"goal": {
				"mass + mass equivalents": {
					"per day": {
						"grams": {
							"fraction string": "3/200"
						}
					}
				}
			}
		}]
	)
'''
import rich

from fractions import Fraction
import copy

import ships.modules.exceptions.parse as parse_exception
import legumes.goals.human.FDA as human_FDA_goal
import apoplast.shows.ingredient_scan_recipe.formulate as formulate_recipe
import apoplast.measures.number.decimal.reduce as reduce_decimal
import apoplast.shows.ingredient_scan.DB.scan.seek as seek_nutrient

#
#	ingredient = "ThiamIn"
#
def find_goal_ingredient (ingredient):
	def for_each (essential):
		try:
			for name in essential ["names"]:
				if (name.lower () == ingredient.lower ()):
					return True;
					
		except Exception:
			pass;
			
		return False

	nutrient = seek_nutrient.presently (
		for_each = for_each
	)
	
	return nutrient;

'''
	using: apoplast ingredients database
	
	find where:
		goal ingredient region == composition ingredient region
'''
def find_goal (
	ingredient_names = [],
	goals_ingredients = [],
	
	records = 0
):
	print ("find goal", ingredient_names)

	for ingredient_name in ingredient_names:
		ingredient_name_lower = ingredient_name.lower ()
		
		print ("	ingredient:", ingredient_name_lower)
		
		try:
			composition_DB_entry_region = find_goal_ingredient (ingredient_name_lower) ["region"]
		except Exception:
			if (records >= 1):
				print ("region not found", ingredient_name_lower)
				
			continue;
		
	
		for goal in goals_ingredients:
			print ("	goal:", goal ["labels"])
		
			for goal_label in goal ["labels"]:
				goal_label = goal_label.lower ()
				print ("	goal_label:", goal_label)
			
				try:
					goal_DB_entry_region = find_goal_ingredient (goal_label) ["region"]
				except Exception:
					if (records >= 1):
						print ("		goal region not found")
						
					pass;
			
				if (
					composition_DB_entry_region ==
					goal_DB_entry_region
				):
					return goal;
			
				
	return None